###################################################################
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
# 
#                         All Rights Reserved
# 
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of Object Craft
# is not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
# 
# OBJECT CRAFT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL OBJECT CRAFT BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE.
# 
###################################################################
import string
from mod_python import util, apache

class Request:
    def __init__(self, req):
        self.__req = req
        self.__fields = util.FieldStorage(self.__req)
        self.__sent_headers = 0

    def has_field(self, name):
        return self.__fields.has_key(name)

    def field_value(self, name):
        field = self.__fields[name]
        if type(field) is type([]):
            value = []
            for elem in field:
                value.append(elem)
            return value
        return field

    def field_names(self):
        return self.__fields.keys()

    def get_uri(self):
        return self.__req.uri

    def get_servername(self):
        return self.__req.connection.server.server_hostname

    def get_header(self, name):
        if self.__req.headers_in.has_key(name):
            return self.__req.headers_in[name]

    def write_header(self, name, value):
        if not self.__sent_headers:
            if string.lower(name) == 'content-type':
                self.__req.content_type = value
            else:
                self.__req.headers_out[name] = value

    def end_headers(self):
        if not self.__sent_headers:
            self.__req.send_http_header()
            self.__sent_headers = 1

    def redirect(self, loc):
        self.write_header('Location', loc)
        return apache.HTTP_MOVED_PERMANENTLY

    def write_content(self, data):
        self.end_headers()
        self.__req.write(data)

    def status(self, num):
        if num == 200:
            return 0
        return num
