###################################################################
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
# 
#                         All Rights Reserved
# 
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of Object Craft
# is not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
# 
# OBJECT CRAFT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL OBJECT CRAFT BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE.
# 
###################################################################

import string, urlparse

from template import Template
from context import CachingTemplateLoaderMixin, PickleSignMixin,\
                    TemplateLoadError, _caller_globals
from app import Application, PageModuleMixin, SimpleAppContext, \
                SessionAppContext
from session import SessionServerAppMixin

class RandomPageModuleMixin(PageModuleMixin):
    def load_page(self, ctx):
        # Get page name from request URI
        uri = ctx.request.get_uri()
        page = ''
        try:
            base_path = urlparse.urlparse(self.base_url())[2]
            uri_path = urlparse.urlparse(uri)[2]
            page = string.split(uri_path, base_path + '/', 1)[1]
        except IndexError:
            pass
        if not page:
            ctx.redirect(self.start_page())

        # Try to load the page module
        self.__page_found = 1
        try:
            self.load_page_module(ctx, page)
        except ImportError:
            ctx.locals.page_name = page
            self.__page_found = 0
        if self.__page_found:
            self.page_enter(ctx)

    def load_badurl_template(self, ctx):
        return Template(ctx, '<internal>',
'''<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
<html>
 <head>
  <title>404 Not Found</title>
 </head>
 <body>
  <h1>Oops!</h1>
   URL <al-value expr="page_name"> was not found on this server.<p>
 </body>
</html>
''')

    def page_enter(self, ctx):
        if self.__page_found:
            return PageModuleMixin.page_enter(self, ctx, ())
        return None

    def process_request(self, ctx):
        if self.__page_found:
            return PageModuleMixin.process_request(self, ctx)
        return None

    def display_response(self, ctx):
        if self.__page_found:
            return PageModuleMixin.display_response(self, ctx)
        else:
            # Display error page
            ctx.write_headers()
            templ = self.load_badurl_template(ctx)
            ctx.set_globals(_caller_globals(3))
            templ.to_html(ctx)
            ctx.flush_content()
            return None

class RandomModularApp(PickleSignMixin,
                       Application,
                       CachingTemplateLoaderMixin,
                       RandomPageModuleMixin):
    def __init__(self, base_url, page_path, start_page, secret):
        PickleSignMixin.__init__(self, secret)
        Application.__init__(self, base_url)
        CachingTemplateLoaderMixin.__init__(self, page_path)
        RandomPageModuleMixin.__init__(self, page_path, start_page)

    def create_context(self):
        return SimpleAppContext(self)

# Same as RandomModularApp with server-side session support
class RandomModularSessionApp(PickleSignMixin,
                              Application,
                              CachingTemplateLoaderMixin,
                              RandomPageModuleMixin,
                              SessionServerAppMixin):
    def __init__(self, base_url, page_path, start_page, secret,
                 session_appid, session_server = 'localhost', server_port = 34343, session_age = 1800):
        PickleSignMixin.__init__(self, secret)
        Application.__init__(self, base_url)
        CachingTemplateLoaderMixin.__init__(self, page_path)
        RandomPageModuleMixin.__init__(self, page_path, start_page)
        SessionServerAppMixin.__init__(self, session_appid, session_server, server_port, session_age)

    def create_context(self):
        return SessionAppContext(self)
