
# Base class for testing Albatross functionality.
#
# Sub-classes are expected to define a class variable "template_dir" which
# tells albatross which directory to search for the html template files.
#
# There is also a class variable, self.ctx, available to classes derived
# from this which contains the current Albatross execution context.
#
# $Id: albatross_test.py,v 1.6 2002/02/21 07:34:21 andrewm Exp $

import unittest
import albatross

import sys
import os

class AlbatrossTestError(Exception):
    pass

class AlbatrossTestCase(unittest.TestCase):
    def setUp(self):
        if not hasattr(self, "template_dir"):
            raise AlbatrossTestError("%s: template_dir not defined in class" % self.__class__)

        # This is because we may not be imported from the current directory
        base_dir = os.path.dirname(sys.modules[__name__].__file__)
        template_dir = os.path.join(base_dir, self.template_dir)

        self.ctx = albatross.SimpleContext(template_dir)

    def tearDown(self):
        pass

    # we use the content_trap to capture and return generated html;
    # html_flush() writes it to stdout directly.
    def gen_html(self, ctx, template):
        ctx.push_content_trap()
        tmpl = ctx.load_template(template)
        tmpl.to_html(ctx)
        return ctx.pop_content_trap()

    def html_test(self, template_file, expect):
        result = self.gen_html(self.ctx, template_file)
        self.assertEqual(result, expect, 'expected "%s", got "%s"' % (expect, result))
